# coding: utf-8

from argparse import ArgumentParser
from datetime import datetime
from random import randint

SYSTEMS = {
    u'ЭДС': 'dou',
    u'ЭШ': 'school',
    u'ЭДО': 'extedu',
    u'ЭПО': 'epo',
    u'ЭК': 'college'

}

MIDDLE_NAMES = [
    u'Артемович',
    u'Алексеевич',
    u'Александрович',
    u'Борисович',
    u'Богданович',
    u'Васильевич',
    u'Владимирович',
    u'Вячеславович',
    u'Генадьевич',
    u'Дмитриевич',
    u'Егорович',
    u'Иванович',
    u'Кириллович',
    u'Леонидович',
    u'Михайлович',
    u'Николаевич',
    u'Олегович',
    u'Петрович',
    u'Романович',
    u'Сергеевич'
]


def param_parser():
    arg_parser = ArgumentParser(
        description=u'Генератор данных для Контингент'
    )
    arg_parser.add_argument(
        '--persons',
        type=int,
        default=1,
        help=u'Количество генерируемых блоков Person.'
             u'По умолчанию будет создан 1 Peron'
    )
    arg_parser.add_argument(
        '--organizations',
        type=int,
        default=0,
        help=u'Количество дополнительно генерируемых блоков Organization. '
             u'По умолчанию будет создана только 1 Organization.'
    )
    arg_parser.add_argument(
        '--region',
        type=str,
        help=u'Регион для создаваемых учреждений.'
             u' Например: Мурманская область.'
    )
    arg_parser.add_argument(
        '--city',
        type=str,
        help=u'Город для создаваемых учреждений.'
             u' Например: г. Мурманск'
    )
    arg_parser.add_argument(
        '--oktmo',
        type=int,
        help=u'Код ОКТМО для создаваемых учреждений.'
    )
    arg_parser.add_argument(
        '--system',
        type=str,
        help=u'ИС-источник формируемых данных.'
             u' Вомзожны варианты: ЭДС, ЭШ, ЭДО, ЭПО, ЭК'
    )
    arg_parser.add_argument(
        '--send',
        type=str,
        default='True',
        help=u'Производить ли отправку данных после формирования.'
             u' По умолчанию = True.'
    )
    opt, _ = arg_parser.parse_known_args()
    opt = vars(opt)
    is_fall, message = check_params(opt)
    if is_fall:
        raise KeyError(message)
    else:
        opt['region'] = opt['region'].decode('utf-8')
        opt['city'] = opt['city'].decode('utf-8')
        opt['system'] = opt['system'].decode('utf-8').upper()

    return opt


def check_params(params):
    is_fall = False
    err_message = ''
    if not params['region']:
        is_fall = True
        err_message = 'Need --region'
    elif not params['city']:
        is_fall = True
        err_message = 'Need --city'
    elif not params['oktmo']:
        is_fall = True
        err_message = 'Need --oktmo'
    elif not params['system']:
        is_fall = True
        err_message = 'Need --system'

    return (is_fall, err_message)


def get_middle_name():
    count = len(MIDDLE_NAMES)-1
    return MIDDLE_NAMES[randint(0, count)]


def get_id():
    return datetime.now().strftime('%y%m%d%H%M%S%f')


class Organization(object):
    def __init__(self, region, city, oktmo):
        self.id = get_id()
        self.region = region
        self.city = city
        self.oktmo = oktmo
        self.inn = self._get_inn()
        self.kpp = randint(9000000, 9999999)
        self.ogrn = self._get_ogrn()
        self.other_id = get_id()
        self.prof_id = get_id()

    @staticmethod
    def _get_inn():
        FACTORS = [2, 4, 10, 3, 5, 9, 4, 6, 8]
        numbers = [randint(1, 9) for _ in xrange(9)]
        str_num = [str(n) for n in numbers]
        numbers = sum([FACTORS[i]*numbers[i] for i in xrange(len(numbers))])
        num_sum = sum([int(n) for n in str(numbers)])
        if num_sum == 10:
            check_sum = '0'
        else:
            check_sum = str(num_sum % 11)
        return ''.join([''.join(str_num), check_sum])

    @staticmethod
    def _get_ogrn():
        num = randint(9100000000, 9999999999)
        return ''.join([str((num*11)+3), '3'])


class Person(object):
    def __init__(
            self, firstname, middlename, lastname,
            delegate_firstname, delegate_middlename,
            delegate_lastname, organization, system
    ):
        self.id = get_id()
        self.firstname = firstname
        self.middlename = middlename
        self.lastname = lastname
        self.snils = self._get_snils()
        self.document_id, self.document_series, \
            self.document_number = self._get_document(system, is_child=True)
        self.decl_id = get_id()
        self.decl_org_id = get_id()
        self.other_id = get_id()
        self.delegate_id = get_id()
        self.delegate_firstname = delegate_firstname
        self.delegate_middlename = delegate_middlename
        self.delegate_lastname = delegate_lastname
        self.delegate_snils = self._get_snils()
        self.delegate_doc_id, self.delegate_doc_series, \
            self.delegate_doc_number = self._get_document(system, is_child=False)
        self.delegate_person_id = get_id()
        self.organization = organization
        self.edu_id = get_id()
        self.project_id = get_id()
        self.org_activities_id = get_id()
        self.activities_id = get_id()
        self.achieve_id = get_id()
        self.ex_activities_id = get_id()
        self.self_stud_id = get_id()
        self.main_edu_class = get_id()
        self.basic_edu_mark_id = get_id()
        self.common_edu_mark_id = get_id()
        self.mid_edu_id = get_id()
        self.ann_mark_id = get_id()
        self.final_mark_id = get_id()
        self.employment_id = get_id()
        self.prof_edu_id = get_id()

    @staticmethod
    def _get_snils():
        numbers = [randint(0, 9) for _ in xrange(9)]
        str_num = [str(n) for n in numbers]
        multipls = list(xrange(1, 10))
        multipls.reverse()
        numbers = [numbers[i]*multipls[i] for i in xrange(len(numbers))]
        num_sum = sum(numbers)
        if num_sum == 100:
            check_sum = '00'
        else:
            check_sum = '%02d' % (num_sum % 101)

        for i in (3, 7):
            str_num.insert(i, '-')

        return ' '.join([''.join(str_num), check_sum])

    @staticmethod
    def _get_document(system, is_child=True):
        doc_id = get_id()
        SERIES_DICT = {
            1: u'V-ТФ',
            2: u'V-АГ'
        }
        if is_child and (system == 'dou' or system == 'school'):
            series = SERIES_DICT[randint(1, 2)]
        else:
            series = int('%04d' % randint(0, 9999))
        number = '%06d'.encode('utf-8') % randint(0, 999999)
        return doc_id, series, number


def create_organization(region, city, oktmo):
    return Organization(region=region, city=city, oktmo=oktmo)


def create_person(fake, organization, system):
    firstname = fake.first_name_male()
    lastname = fake.last_name_male()
    middlename = get_middle_name()
    del_firstname = fake.first_name_male()
    del_lastname = lastname
    del_middlename = get_middle_name()

    return Person(
        firstname, middlename, lastname,
        del_firstname, del_middlename, del_lastname,
        organization, system
    )